From 199c2623939333bf01d8e5fc4eca00587a9c15dd Mon Sep 17 00:00:00 2001
From: Craig Ringer <craig@2ndquadrant.com>
Date: Tue, 1 Mar 2016 20:57:17 +0800
Subject: [PATCH] Tests for logical decoding timeline following

---
 src/test/recovery/Makefile                         |  2 +
 .../recovery/t/006_logical_decoding_timelines.pl   | 96 ++++++++++++++++++++++
 2 files changed, 98 insertions(+)
 create mode 100644 src/test/recovery/t/006_logical_decoding_timelines.pl

diff --git a/src/test/recovery/Makefile b/src/test/recovery/Makefile
index 330ab2b..ad1d764 100644
--- a/src/test/recovery/Makefile
+++ b/src/test/recovery/Makefile
@@ -9,6 +9,8 @@
 #
 #-------------------------------------------------------------------------
 
+EXTRA_INSTALL=contrib/test_decoding
+
 subdir = src/test/recovery
 top_builddir = ../../..
 include $(top_builddir)/src/Makefile.global
diff --git a/src/test/recovery/t/006_logical_decoding_timelines.pl b/src/test/recovery/t/006_logical_decoding_timelines.pl
new file mode 100644
index 0000000..e3acaa9
--- /dev/null
+++ b/src/test/recovery/t/006_logical_decoding_timelines.pl
@@ -0,0 +1,96 @@
+# Demonstrate that logical can follow timeline switches.
+#
+# Logical replication slots can follow timeline switches but it's
+# normally not possible to have a logical slot on a replica where
+# promotion and a timeline switch can occur. The only ways
+# we can create that circumstance are:
+#
+# * By doing a filesystem-level copy of the DB, since pg_basebackup
+#   excludes pg_replslot but we can copy it directly; or
+#
+# * by creating a slot directly at the C level on the replica and
+#   advancing it as we go using the low level APIs. It can't be done
+#   from SQL since logical decoding isn't allowed on replicas.
+#
+# This module uses the first approach to show that timeline following
+# on a logical slot works.
+#
+use strict;
+use warnings;
+use PostgresNode;
+use TestLib;
+use Test::More tests => 3;
+use RecursiveCopy;
+
+my ( $stdout, $stderr, $ret );
+
+# Initialize master node
+my $node_master = get_new_node('master');
+$node_master->init( allows_streaming => 1, has_archiving => 1 );
+$node_master->append_conf( 'postgresql.conf', "wal_level = 'logical'\n" );
+$node_master->append_conf( 'postgresql.conf', "max_replication_slots = 2\n" );
+$node_master->append_conf( 'postgresql.conf', "max_wal_senders = 2\n" );
+$node_master->append_conf( 'postgresql.conf', "log_min_messages = 'debug2'\n" );
+$node_master->dump_info;
+$node_master->start;
+
+$node_master->psql_check( 'postgres',
+"SELECT pg_create_logical_replication_slot('before_basebackup', 'test_decoding');"
+);
+$node_master->psql_check( 'postgres', "CREATE TABLE decoding(blah text);" );
+$node_master->psql_check( 'postgres',
+    "INSERT INTO decoding(blah) VALUES ('afterbb');" );
+$node_master->psql_check( 'postgres', 'CHECKPOINT;' );
+
+my $backup_name = 'b1';
+$node_master->backup_fs_hot($backup_name);
+
+my $node_replica = get_new_node('replica');
+$node_replica->init_from_backup(
+    $node_master, $backup_name,
+    has_streaming => 1,
+    has_restoring => 1
+);
+$node_replica->start;
+
+$node_master->psql_check( 'postgres',
+"SELECT pg_create_logical_replication_slot('after_basebackup', 'test_decoding');"
+);
+$node_master->psql_check( 'postgres',
+    "INSERT INTO decoding(blah) VALUES ('afterbb');" );
+$node_master->psql_check( 'postgres', 'CHECKPOINT;' );
+
+# Boom, crash
+$node_master->stop('immediate');
+
+$node_replica->promote;
+$node_replica->poll_query_until( 'postgres',
+    "SELECT NOT pg_is_in_recovery();" );
+
+$node_replica->psql_check( 'postgres',
+    "INSERT INTO decoding(blah) VALUES ('after failover');" );
+
+# Shouldn't be able to read from slot created after base backup
+$ret = $node_replica->psql_expert(
+    'postgres',
+"SELECT * FROM pg_logical_slot_peek_changes('after_basebackup', NULL, NULL);",
+    stdout => \$stdout,
+    stderr => \$stderr
+);
+is( $ret, 3, 'replaying from after_basebackup slot fails' );
+like(
+    $stderr,
+    qr/replication slot "after_basebackup" does not exist/,
+    'after_basebackup slot missing'
+);
+
+# Should be able to read from slot created before base backup
+diag "Trying to replay from slot before_basebackup, timeout in 30s";
+$node_replica->psql_check(
+    'postgres',
+"SELECT * FROM pg_logical_slot_peek_changes('before_basebackup', NULL, NULL);",
+    timeout => 30
+);
+ok('replay from before_basebackup successful');
+
+1;
-- 
2.1.0

